# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core

QtCore.QParallelAnimationGroup.__bases__ = (core.AnimationGroup,)


class ParallelAnimationGroup(QtCore.QParallelAnimationGroup):
    pass



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtWidgets

from prettyqt import widgets


QtWidgets.QTreeWidget.__bases__ = (widgets.TreeView,)


class TreeWidget(QtWidgets.QTreeWidget):
    def sort(self, column=0, reverse=False):
        order = QtCore.Qt.DescendingOrder if reverse else QtCore.Qt.AscendingOrder
        self.sortItems(column, order)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = TreeWidget()
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import core


class Transform(QtGui.QTransform):
    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)

    def __getitem__(self, value) -> float:
        if value[0] == 0:
            if value[1] == 0:
                return self.m11()
            elif value[1] == 1:
                return self.m12()
            elif value[1] == 2:
                return self.m13()
        elif value[0] == 1:
            if value[1] == 0:
                return self.m21()
            elif value[1] == 1:
                return self.m22()
            elif value[1] == 2:
                return self.m23()
        elif value[0] == 2:
            if value[1] == 0:
                return self.m31()
            elif value[1] == 1:
                return self.m32()
            elif value[1] == 2:
                return self.m33()
        raise ValueError(f"Wrong value {value}")


if __name__ == "__main__":
    transform = Transform()



# -*- coding: utf-8 -*-

from qtpy import QtCore, QtGui

from prettyqt.utils import bidict


SEQUENCE_MATCHES = bidict(
    none=QtGui.QKeySequence.NoMatch,
    partial=QtGui.QKeySequence.PartialMatch,
    exact=QtGui.QKeySequence.ExactMatch,
)

MODS = {
    QtCore.Qt.ShiftModifier: QtCore.Qt.SHIFT,
    QtCore.Qt.ControlModifier: QtCore.Qt.CTRL,
    QtCore.Qt.AltModifier: QtCore.Qt.ALT,
    QtCore.Qt.MetaModifier: QtCore.Qt.META,
}


class KeySequence(QtGui.QKeySequence):
    def __str__(self):
        return self.toString()

    def __repr__(self):
        return f"KeySequence({self.toString()!r})"

    def __bool__(self):
        return not self.isEmpty()

    def __reduce__(self):
        return (self.__class__, (self.toString(),))

    def get_matches(self, seq):
        if isinstance(seq, str):
            seq = KeySequence(seq)
        return SEQUENCE_MATCHES.inv[self.matches(seq)]

    @classmethod
    def to_shortcut_str(cls, key, mod=0):
        for k, v in MODS.items():
            if mod & k:
                key += v
        return str(cls(key))



# -*- coding: utf-8 -*-

import contextlib

from qtpy import QtCore

from prettyqt import core


QtCore.QAbstractState.__bases__ = (core.Object,)


class AbstractState(QtCore.QAbstractState):
    def serialize_fields(self):
        return dict(active=self.active())

    @contextlib.contextmanager
    def on_state_active(self):
        self.onEntry()
        yield self
        self.onExit()


if __name__ == "__main__":
    state = AbstractState()



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import core, gui


class Brush(QtGui.QBrush):
    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)

    def get_texture_image(self):
        return gui.Image(self.textureImage())



# -*- coding: utf-8 -*-

from qtpy import QtCore

from prettyqt import core


QtCore.QAbstractProxyModel.__bases__ = (core.AbstractItemModel,)


class AbstractProxyModel(QtCore.QAbstractProxyModel):
    pass



# -*- coding: utf-8 -*-

from typing import Tuple, Optional

from qtpy import QtWidgets

from prettyqt import widgets, constants
from prettyqt.utils import bidict


ALIGNMENTS = bidict(
    left=constants.ALIGN_LEFT,
    right=constants.ALIGN_RIGHT,
    top=constants.ALIGN_TOP,
    bottom=constants.ALIGN_BOTTOM,
    top_left=constants.ALIGN_TOP_LEFT,
    top_right=constants.ALIGN_TOP_RIGHT,
    bottom_left=constants.ALIGN_BOTTOM_LEFT,
    bottom_right=constants.ALIGN_BOTTOM_RIGHT,
    center=constants.ALIGN_CENTER,
)

QtWidgets.QGraphicsGridLayout.__bases__ = (widgets.GraphicsLayout,)


class GraphicsGridLayout(QtWidgets.QGraphicsGridLayout):
    def __getitem__(
        self, idx: Tuple[int, int]
    ) -> Optional[QtWidgets.QGraphicsLayoutItem]:
        return self.itemAt(*idx)

    def __setitem__(self, idx, value):
        row, col = idx
        rowspan = row.stop - row.start + 1 if isinstance(row, slice) else 1
        colspan = col.stop - col.start + 1 if isinstance(col, slice) else 1
        rowstart = row.start if isinstance(row, slice) else row
        colstart = col.start if isinstance(col, slice) else col
        self.add(value, rowstart, colstart, rowspan, colspan)

    def serialize_fields(self):
        items = []
        positions = []
        for row in self.rowCount():
            for col in self.columnCount():
                item = self.itemAt(row, col)
                if item is not None:
                    items.append(item)
                    positions.append((row, col))
        return dict(widgets=widgets, positions=positions)

    def __setstate__(self, state):
        self.__init__()
        for i, (item, pos) in enumerate(zip(state["widgets"], state["positions"])):
            x, y, w, h = pos
            self[x : x + w - 1, y : y + h - 1] = item

    def __iter__(self):
        return iter(self[i] for i in range(self.count()) if self[i] is not None)

    def __add__(self, other):
        if isinstance(other, (tuple, list)):
            for i, control in enumerate(other):
                self[self.rowCount(), i] = other
        else:
            self[self.rowCount(), 0 : self.columnCount() - 1] = other
        return self

    def add(self, item, rowstart, colstart, rowspan=1, colspan=1):
        if isinstance(item, QtWidgets.QWidget):
            self.addWidget(item, rowstart, colstart, rowspan, colspan)
        elif isinstance(item, QtWidgets.QLayout):
            self.addLayout(item, rowstart, colstart, rowspan, colspan)
        else:
            self.addItem(item, rowstart, colstart, rowspan, colspan)

    def append(self, item):
        self[self.rowCount(), 0 : self.columnCount() - 1] = item

    def set_column_alignment(self, column: int, alignment: str):
        self.setColumnAlignment(column, ALIGNMENTS[alignment])

    def set_row_alignment(self, row: int, alignment: str):
        self.setRowAlignment(row, ALIGNMENTS[alignment])


if __name__ == "__main__":
    app = widgets.app()
    layout = GraphicsGridLayout()
    layout[1, 5:6] = widgets.RadioButton("1 2 3 jk jkjl j kföldsjfköj")
    layout[3:5, 7:8] = widgets.RadioButton("2")
    layout[3:5, 1:4] = widgets.RadioButton("3")
    layout += widgets.RadioButton("3")
    layout += widgets.RadioButton("4")
    widget = widgets.Widget()
    widget.set_layout(layout)
    print(layout)
    widget.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui

STYLE_HINTS = gui.font.STYLE_HINTS  # type: ignore


class FontInfo(QtGui.QFontInfo):
    def get_style_hint(self):
        return STYLE_HINTS.inv[self.styleHint()]



# -*- coding: utf-8 -*-

from typing import Optional, Union, DefaultDict, List
from collections import defaultdict
from contextlib import contextmanager
import itertools

from qtpy import QtCore, QtWidgets

from prettyqt import core

counter_dict: DefaultDict = defaultdict(itertools.count)


class Object(QtCore.QObject):
    def serialize_fields(self):
        return dict(object_name=self.objectName())

    def __setstate__(self, state):
        self.__init__()
        self.set_id(state["object_name"])

    def __getstate__(self):
        return self.serialize()

    def serialize(self):
        classes = type(self).mro()
        dct = dict()
        for klass in reversed(classes):
            if "serialize_fields" in klass.__dict__:
                data = klass.serialize_fields(self)
                dct.update(data)
        return dct

    @contextmanager
    def block_signals(self):
        self.blockSignals(True)
        yield None
        self.blockSignals(False)

    def to_json(self):
        dct = self.__getstate__()
        for k, v in dct.items():
            if isinstance(v, QtCore.QObject):
                dct[k] = v.to_json()
        return dct

    def set_unique_id(self):
        class_name = type(self).__name__
        count = next(counter_dict[class_name])
        self.set_id(f"{class_name}_{count}")

    def set_id(self, name: str):
        self.setObjectName(name)

    def get_id(self) -> str:
        return self.objectName()

    @property
    def id(self) -> str:
        return self.objectName()

    @id.setter
    def id(self, name: str):
        self.setObjectName(name)

    def find_children(
        self,
        typ=QtCore.QObject,
        name: Optional[Union[str, QtCore.QRegularExpression]] = None,
        recursive: bool = True,
    ) -> List[QtCore.QObject]:
        if recursive:
            flag = QtCore.Qt.FindChildrenRecursively
        else:
            flag = QtCore.Qt.FindDirectChildrenOnly
        return self.findChildren(typ, name=name, options=flag)

    def find_child(
        self,
        typ=QtCore.QObject,
        name: Optional[Union[str, QtCore.QRegularExpression]] = None,
        recursive: bool = True,
    ):
        if recursive:
            flag = QtCore.Qt.FindChildrenRecursively
        else:
            flag = QtCore.Qt.FindDirectChildrenOnly
        return self.findChild(typ, name, options=flag)

    def store_widget_states(self, settings=None, key: str = "states"):
        splitters = self.find_children(QtWidgets.QSplitter)
        splitter_dct = {
            sp.objectName(): sp.saveState() for sp in splitters if sp.objectName()
        }
        mainwindows = self.find_children(QtWidgets.QMainWindow)
        mw_dct = {
            mw.objectName(): mw.saveState() for mw in mainwindows if mw.objectName()
        }
        headerviews = self.find_children(QtWidgets.QHeaderView)
        headerview_dct = {
            h.objectName(): h.saveState() for h in headerviews if h.objectName()
        }
        settings = core.Settings() if settings is None else settings
        settings[key] = dict(
            splitters=splitter_dct, mainwindows=mw_dct, headerviews=headerview_dct
        )

    def restore_widget_states(self, settings=None, key: str = "states"):
        settings = core.Settings() if settings is None else settings
        splitters = settings[key].get("splitters")
        mainwindows = settings[key].get("splitters")
        headerviews = settings[key].get("headerviews")
        if splitters is not None:
            for k, v in splitters.items():
                w = self.find_child(QtWidgets.QSplitter, name=k)
                if w is not None:
                    w.restoreState(v)
        if mainwindows is not None:
            for k, v in mainwindows.items():
                w = self.find_child(QtWidgets.QMainWindow, name=k)
                if w is not None:
                    w.restoreState(v)
        if headerviews is not None:
            for k, v in headerviews.items():
                w = self.find_child(QtWidgets.QHeaderView, name=k)
                if w is not None:
                    w.restoreState(v)

    def find_parent(
        self, typ: QtCore.QObject, name: Optional[str] = None
    ) -> Optional[QtCore.QObject]:
        node = self
        while node:
            node = node.parent()
            if isinstance(node, typ):
                if name is None or node.objectName() == name:
                    return node
        return None

    def raise_dock(self) -> bool:
        w = self.find_parent(QtWidgets.QDockWidget)
        if w is None:
            return False
        w.setVisible(True)
        w.raise_()
        return True



# -*- coding: utf-8 -*-

"""Network module.

Contains QtNetWork-based classes
"""

from .networkcookie import NetworkCookie
from .networkcookiejar import NetworkCookieJar
from .networkrequest import NetworkRequest
from .networkaccessmanager import NetworkAccessManager

__all__ = [
    "NetworkCookie",
    "NetworkCookieJar",
    "NetworkRequest",
    "NetworkAccessManager",
]



# -*- coding: utf-8 -*-

import pathlib
from typing import Union

from qtpy import QtGui

from prettyqt import gui, core


QtGui.QPixmap.__bases__ = (gui.PaintDevice,)


class Pixmap(QtGui.QPixmap):
    def __bool__(self):
        return not self.isNull()

    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)

    @classmethod
    def from_file(cls, path: Union[pathlib.Path, str]):
        if isinstance(path, str):
            path = pathlib.Path(path)
        with path.open(mode="rb") as f:
            data = f.read()
        # Create widget
        pixmap = cls()
        pixmap.loadFromData(data)
        return pixmap


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    p = Pixmap()
    print(bool(p))



# -*- coding: utf-8 -*-

from qtpy import QtGui

from prettyqt import gui, core


QtGui.QPicture.__bases__ = (gui.PaintDevice,)


class Picture(QtGui.QPicture):
    def __getstate__(self):
        return core.DataStream.create_bytearray(self)

    def __setstate__(self, ba):
        self.__init__()
        core.DataStream.write_bytearray(ba, self)



from typing import List

from qtpy import QtWidgets, QtCore

from prettyqt import core, widgets, gui, constants
from prettyqt.utils import InvalidParamError, bidict, helpers


ALIGNMENTS = bidict(
    left=constants.ALIGN_LEFT,
    right=constants.ALIGN_RIGHT,
    top=constants.ALIGN_TOP,
    bottom=constants.ALIGN_BOTTOM,
    top_left=constants.ALIGN_TOP_LEFT,
    top_right=constants.ALIGN_TOP_RIGHT,
    bottom_left=constants.ALIGN_BOTTOM_LEFT,
    bottom_right=constants.ALIGN_BOTTOM_RIGHT,
    center=constants.ALIGN_CENTER,
)

DRAG_MODES = bidict(
    none=QtWidgets.QGraphicsView.NoDrag,
    scroll_hand=QtWidgets.QGraphicsView.ScrollHandDrag,
    rubber_band=QtWidgets.QGraphicsView.RubberBandDrag,
)

CACHE_MODES = bidict(
    none=QtWidgets.QGraphicsView.CacheNone,
    background=QtWidgets.QGraphicsView.CacheBackground,
)

OPTIMIZATION_FLAGS = bidict(
    dont_clip_painter=QtWidgets.QGraphicsView.DontClipPainter,
    dont_save_painter_state=QtWidgets.QGraphicsView.DontSavePainterState,
    dont_adjust_for_antialiasing=QtWidgets.QGraphicsView.DontAdjustForAntialiasing,
)

VIEWPORT_ANCHORS = bidict(
    none=QtWidgets.QGraphicsView.NoAnchor,
    view_center=QtWidgets.QGraphicsView.AnchorViewCenter,
    under_mouse=QtWidgets.QGraphicsView.AnchorUnderMouse,
)

VIEWPORT_UPDATE_MODES = bidict(
    full=QtWidgets.QGraphicsView.FullViewportUpdate,
    minimal=QtWidgets.QGraphicsView.MinimalViewportUpdate,
    smart=QtWidgets.QGraphicsView.SmartViewportUpdate,
    bounding_rect=QtWidgets.QGraphicsView.BoundingRectViewportUpdate,
    none=QtWidgets.QGraphicsView.NoViewportUpdate,
)

ITEM_SELECTION_MODES = bidict(
    contains_shape=QtCore.Qt.ContainsItemShape,
    intersects_shape=QtCore.Qt.IntersectsItemShape,
    contains_bounding_rect=QtCore.Qt.ContainsItemBoundingRect,
    intersects_bounding_rect=QtCore.Qt.IntersectsItemBoundingRect,
)

SCENE_LAYERS = widgets.graphicsscene.SCENE_LAYERS  # type: ignore

RENDER_HINTS = gui.painter.RENDER_HINTS  # type: ignore

QtWidgets.QGraphicsView.__bases__ = (widgets.AbstractScrollArea,)


class GraphicsView(QtWidgets.QGraphicsView):
    def serialize_fields(self):
        return dict(
            scene=self.scene(),
            background_brush=gui.Brush(self.backgroundBrush()),
            foreground_brush=gui.Brush(self.foregroundBrush()),
            transformation_anchor=self.get_transformation_anchor(),
            resize_anchor=self.get_resize_anchor(),
            viewport_update_mode=self.get_viewport_update_mode(),
            drag_mode=self.get_drag_mode(),
            rubberband_selection_mode=self.get_rubberband_selection_mode(),
            scene_rect=core.RectF(self.sceneRect()),
            cache_mode=self.get_cache_mode(),
            is_interactive=self.isInteractive(),
        )

    def __getitem__(self, index: int):
        return self.items()[index]

    def invalidate_scene(self, rect: QtCore.QRectF, layer: str = "all"):
        if layer not in SCENE_LAYERS:
            raise InvalidParamError(layer, SCENE_LAYERS)
        self.invalidateScene(rect, SCENE_LAYERS[layer])

    def set_transformation_anchor(self, mode: str):
        """Set how the view should position the scene during transformations.

        Allowed values are "none", "view_center", "under_mouse"

        Args:
            mode: transformation anchor to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in VIEWPORT_ANCHORS:
            raise InvalidParamError(mode, VIEWPORT_ANCHORS)
        self.setTransformationAnchor(VIEWPORT_ANCHORS[mode])

    def get_transformation_anchor(self) -> str:
        """Return current transformation anchor.

        Possible values: "none", "view_center", "under_mouse"

        Returns:
            viewport anchor
        """
        return VIEWPORT_ANCHORS.inv[self.transformationAnchor()]

    def set_resize_anchor(self, mode: str):
        """Set how the view should position the scene during resizes.

        Allowed values are "none", "view_center", "under_mouse"

        Args:
            mode: resize anchor to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in VIEWPORT_ANCHORS:
            raise InvalidParamError(mode, VIEWPORT_ANCHORS)
        self.setResizeAnchor(VIEWPORT_ANCHORS[mode])

    def get_resize_anchor(self) -> str:
        """Return current resize anchor.

        Possible values: "none", "view_center", "under_mouse"

        Returns:
            resize anchor
        """
        return VIEWPORT_ANCHORS.inv[self.resizeAnchor()]

    def set_viewport_update_mode(self, mode: str):
        """Set how the viewport should update its contents.

        Allowed values are "full", "minimal", "smart", "bounding_rect", "none"

        Args:
            mode: viewport update mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in VIEWPORT_UPDATE_MODES:
            raise InvalidParamError(mode, VIEWPORT_UPDATE_MODES)
        self.setViewportUpdateMode(VIEWPORT_UPDATE_MODES[mode])

    def get_viewport_update_mode(self) -> str:
        """Return current viewport update mode.

        Possible values: "full", "minimal", "smart", "bounding_rect", "none"

        Returns:
            viewport update mode
        """
        return VIEWPORT_UPDATE_MODES.inv[self.viewportUpdateMode()]

    def set_drag_mode(self, mode: str):
        """Set the behavior for dragging the mouse while the left mouse button is pressed.

        Allowed values are "none", "scroll_hand", "rubber_band"

        Args:
            mode: drag mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in DRAG_MODES:
            raise InvalidParamError(mode, DRAG_MODES)
        self.setDragMode(DRAG_MODES[mode])

    def get_drag_mode(self) -> str:
        """Return current drag mode.

        Possible values: "none", "scroll_hand", "rubber_band"

        Returns:
            drag mode
        """
        return DRAG_MODES.inv[self.dragMode()]

    def set_rubberband_selection_mode(self, mode: str):
        """Set the behavior for selecting items with a rubber band selection rectangle.

        Allowed values are "contains_shape", "intersects_shape", "contains_bounding_rect",
        "intersects_bounding_rect"

        Args:
            mode: rubberband selection mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in ITEM_SELECTION_MODES:
            raise InvalidParamError(mode, ITEM_SELECTION_MODES)
        self.setRubberBandSelectionMode(ITEM_SELECTION_MODES[mode])

    def get_rubberband_selection_mode(self) -> str:
        """Return current rubberband selection mode.

        Possible values: "contains_shape", "intersects_shape", "contains_bounding_rect",
        "intersects_bounding_rect"

        Returns:
            rubberband selection mode
        """
        return ITEM_SELECTION_MODES.inv[self.rubberBandSelectionMode()]

    def set_cache_mode(self, mode: str):
        """Set the cache mode.

        Allowed values are "none", "background"

        Args:
            mode: cache mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in CACHE_MODES:
            raise InvalidParamError(mode, CACHE_MODES)
        self.setCacheMode(CACHE_MODES[mode])

    def get_cache_mode(self) -> str:
        """Return current cache mode.

        Possible values: "none", "background"

        Returns:
            cache mode
        """
        return CACHE_MODES.inv[self.cacheMode()]

    def set_optimization_flags(self, *items: str):
        for item in items:
            if item not in OPTIMIZATION_FLAGS:
                raise InvalidParamError(item, OPTIMIZATION_FLAGS)
        flags = helpers.merge_flags(items, OPTIMIZATION_FLAGS)
        self.setOptimizationFlags(flags)

    def get_optimization_flags(self) -> List[str]:
        return [k for k, v in OPTIMIZATION_FLAGS.items() if v & self.optimizationFlags()]


if __name__ == "__main__":
    app = widgets.app()
    view = GraphicsView()
    scene = widgets.GraphicsScene()
    scene.add_line(core.Line(0, 0, 200, 100))
    view.setScene(scene)
    view.show()
    app.main_loop()



# -*- coding: utf-8 -*-

from typing import Optional

from qtpy import QtWidgets

from prettyqt import widgets, constants


QtWidgets.QTreeView.__bases__ = (widgets.AbstractItemView,)


class TreeView(QtWidgets.QTreeView):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        class_name = type(self).__name__
        self.set_id(class_name)
        # visual settings
        self.setAnimated(True)
        self.setRootIsDecorated(False)
        self.setAllColumnsShowFocus(True)
        self.setUniformRowHeights(True)
        self.setAlternatingRowColors(True)
        self.setWordWrap(False)

        # misc
        self.h_header = widgets.HeaderView("horizontal", parent=self)
        self.set_selection_mode("extended")

    @property
    def h_header(self):
        return self.header()

    @h_header.setter
    def h_header(self, header):
        self.setHeader(header)

    def expand_all(self):
        self.expandAll()

    def set_indentation(self, indentation: int):
        self.setIndentation(indentation)

    def setup_list_style(self):
        self.setSelectionBehavior(self.SelectRows)
        self.h_header.setStretchLastSection(True)

    def adapt_sizes(self):
        model = self.model()
        if model is not None and (model.rowCount() * model.columnCount()) < 1000:
            self.h_header.resizeSections(self.h_header.ResizeToContents)
        else:
            self.h_header.resize_sections("interactive")

    def sort_by_column(self, column: Optional[int], ascending: bool = True):
        column = -1 if column is None else column
        order = constants.ASCENDING if ascending else constants.DESCENDING
        self.sortByColumn(column, order)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    dlg = QtWidgets.QMainWindow()
    status_bar = TreeView()
    dlg.show()
    app.main_loop()
